@@ -1,6 +1,7 @@
 package HTML::Mason::CGIHandler;
 
 use strict;
+use warnings;
 
 use HTML::Mason;
 use HTML::Mason::Utils;
@@ -36,7 +37,7 @@ __PACKAGE__->contained_objects
     (
      interp => 'HTML::Mason::Interp',
      cgi_request => { class   => 'HTML::Mason::FakeApache', # $r
-		      delayed => 1 },
+                      delayed => 1 },
     );
 
 
@@ -45,15 +46,11 @@ sub new {
 
     my %p = @_;
     my $self = $package->SUPER::new(comp_root => $ENV{DOCUMENT_ROOT},
-				    request_class => 'HTML::Mason::Request::CGI',
-				    error_mode => 'output',
-				    error_format => 'html',
-				    %p);
+                                    request_class => 'HTML::Mason::Request::CGI',
+                                    error_mode => 'output',
+                                    error_format => 'html',
+                                    %p);
 
-    # If the user passed an out_method parameter, then we don't want
-    # to overwrite it later.  This doesn't handle the case where the
-    # user creates their own Interp object with a custom out_method
-    # and passes that to this method, though (grrr).
     $self->{has_custom_out_method} = $p{out_method} ? 1 : 0;
 
     $self->interp->compiler->add_allowed_globals('$r');
@@ -74,8 +71,8 @@ sub handle_comp {
 sub handle_cgi_object {
     my ($self, $cgi) = (shift, shift);
     $self->_handler( { comp => $cgi->path_info,
-		       cgi       => $cgi },
-		     @_);
+                       cgi       => $cgi },
+                     @_);
 }
 
 sub _handler {
@@ -92,14 +89,11 @@ sub _handler {
         my $sent_headers = 0;
 
         my $out_method = sub {
-
             # Send headers if they have not been sent by us or by user.
             # We use instance here because if we store $request we get a
             # circular reference and a big memory leak.
             if (!$sent_headers and HTML::Mason::Request->instance->auto_send_headers) {
-                unless ($r->http_header_sent) {
-                    $r->send_http_header();
-                }
+                $r->send_http_header();
                 $sent_headers = 1;
             }
 
@@ -128,23 +122,22 @@ sub _handler {
     }
 
     if (my $err = $@) {
-	my $retval = isa_mason_exception($err, 'Abort')   ? $err->aborted_value  :
-		     isa_mason_exception($err, 'Decline') ? $err->declined_value :
-		     rethrow_exception $err;
-
+        my $retval = isa_mason_exception($err, 'Abort')   ? $err->aborted_value  :
+                     isa_mason_exception($err, 'Decline') ? $err->declined_value :
+                     rethrow_exception $err;
 
         # Unlike under mod_perl, we cannot simply return a 301 or 302
         # status and let Apache send headers, we need to explicitly
         # send this header ourself.
         $r->send_http_header if $retval && grep { $retval eq $_ } ( 200, 301, 302 );
 
-	return $retval;
+        return $retval;
     }
 
     if (@_) {
-	# This is a secret feature, and should stay secret (or go
-	# away) because it's just a hack for the test suite.
-	$_[0] .= $r->http_header . $self->{output};
+        # This is a secret feature, and should stay secret (or go
+        # away) because it's just a hack for the test suite.
+        $_[0] .= $r->http_header . $self->{output};
     }
 
     return wantarray ? @result : defined wantarray ? $result[0] : undef;
@@ -207,12 +200,13 @@ sub exec
     # if we are the top-level request. Since the out_method sends
     # headers, this will typically only apply after $m->abort.
     if (!$self->is_subrequest
-	and $self->auto_send_headers
-	and !$r->http_header_sent
-	and (!$retval or $retval==200)) {
-	$r->send_http_header();
+        and $self->auto_send_headers
+        and !$r->http_header_sent
+        and (!$retval or $retval==200)) {
+        $r->send_http_header();
     }
 
+    return $retval;
 }
 
 sub redirect {
@@ -239,10 +233,17 @@ HTML::Mason::CGIHandler - Use Mason in a CGI environment
 
 In httpd.conf or .htaccess:
 
-   Action html-mason /cgi-bin/mason_handler.cgi
-   <LocationMatch "\.html$">
-    SetHandler html-mason
-   </LocationMatch>
+    <LocationMatch "\.html$">
+        Action html-mason /cgi-bin/mason_handler.cgi
+        AddHandler html-mason .html
+    </LocationMatch>
+    <LocationMatch "^/cgi-bin/">
+        RemoveHandler .html
+    </LocationMatch>
+    <FilesMatch "(autohandler|dhandler)$">
+        Order allow,deny
+        Deny from all
+    </FilesMatch>
 
 A script at /cgi-bin/mason_handler.pl :
 
@@ -343,20 +344,6 @@ Interpreter lasts for the entire lifetime of the handler.
 
 =back
 
-=head2 Calling abort() Under CGIHandler
-
-If you call C<< $m->abort() >> under CGIHandler, request execution
-stops, just as when running ApacheHandler.  For compatibility with
-ApacheHandler, if you call C<< $m->abort() >> with no argument, or
-with an argument of 0 or 200 (OK statuses), then headers will be sent.
-Otherwise, they are not.  However, unlike with ApacheHandler, calling
-C<< $m->abort() >> with an HTTP status code does not cause the web
-server to return that status code to the browser.
-
-It would be theoretically possible to use CGI's "Status:" header to
-set the status in response to the top-level component's return value
-or the aborted value. We will try this in a later version.
-
 =head2 $r Methods
 
 =over 4
